#!/usr/bin/env python
# -*- coding:utf-8 -*-
import socket
import threading
import tkinter as tk
from tkinter import ttk

if __name__ == '__main__':
    from udp import UDP
else:
    from .udp import UDP


class Form(tk.Tk):
    def __init__(self, *args, **kwargs):
        """
        Argument:
            dialog=<class 'bool'>
            icon=full_path
            hasmenu=True or False: Change height.
            master=object
            minsize=(width, height)
            name=<class "str"> 
            position=[width, height, left, top]
            title=<class "str">
            topmost=False
            udp_port=4000
            udp_bufsize=4096
        """
        if kwargs.get('toplevel', False):
            # Use for dialog new window.
            tk.Toplevel.__init__(self)
            self.resizable(0, 0)
            self.grab_set()
            # Focus to me.
            self.focus_set()
            # Not show taskbar.
            self.transient(self.master)
        else:
            super().__init__(
                #border=0,
                #relief=None,
            )

        if len(kwargs.get('minsize', ())) == 2:
            self.minsize(
                kwargs.get('minsize')[0],
                kwargs.get('minsize')[1])

        self.border_style = {'borderwidth': 0, 'relief': 'solid'}

        self._scaling = self._get_dpi(self) / 72
        self.tk.call('tk', 'scaling', self.scaling)
        #self.tk.call('tk', 'scaling', 0.5)

        # 96 is development PC's DPI. This value is default.
        self._screen_ratio = 96 / self._get_dpi(self)

        self.master = kwargs.get('master', None)
        self.title(kwargs.get('title', ''))
        self.name = kwargs.get('name', '')

        self._hasmenu = kwargs.get('hasmenu', False)
        if len(kwargs.get('position', [])) == 4:
            width = int(kwargs.get('position')[0])
            if self.minsize()[0] > width:
                width = self.minsize()[0]

            height = int(kwargs.get('position')[1])
            if self.minsize()[1] > height:
                height = self.minsize()[1]

            x = kwargs.get('position')[2]
            y = kwargs.get('position')[3]

            self.position = [width, height, x, y]

        if kwargs.get('icon', '') != '':
            self.icon = kwargs.get('icon', '')

        if kwargs.get('topmost', False):
            self.grab_set()

        if kwargs.get('dialog', False):
            self.resizable(0,0)

        self._attach_events()

        if kwargs.get('udp_port', '') != '':
            self.udp = UDP(self, *args, **kwargs)
            self.udp.receive.trace('w', self.udp_receive_changed)

        self.resize_event = True

    @property
    def host(self):
        result = socket.gethostbyname_ex(socket.gethostname())
        result = result[2][0]
        return result

    @property
    def icon(self):
        return self._icon
    @icon.setter
    def icon(self, value):
        self._icon = value
        self.tk.call('wm', 'iconphoto', self._w, tk.PhotoImage(file=self._icon))

    @property
    def position(self):
        return [
            self.winfo_width(),
            self.winfo_height(),
            self.winfo_x(),
            self.winfo_y()]
    @position.setter
    def position(self, value):
        """
        geometry=[width, height, left, top]
        """
        if self._hasmenu:
            value[1] += 20
        self.geometry(f'{value[0]}x{value[1]}+{value[2]}+{value[3]}')

    @property
    def resize_event(self):
        return self._resize_event
    @resize_event.setter
    def resize_event(self, value):
        self._resize_event = value
        if self._resize_event:
            self.bind('<Configure>', self._on_resize)
        else:
            self.unbind('<Configure>')

    @property
    def scaling(self):
        return self._scaling

    @property
    def screen_ratio(self):
        return self._screen_ratio

    def _attach_events(self):
        self.protocol('WM_DELETE_WINDOW', self.on_exit)

        self.bind('<Configure>', self._on_resize)
        self.unbind('<Configure>')

        self.bind('<MouseWheel>', self._mouse_wheel)
        self.bind('<Motion>', self._mouse_move)

        self.bind('<ButtonPress-1>', self._button1_press)
        self.bind('<ButtonRelease-1>', self._button1_release)

        self.bind('<ButtonPress-2>', self._button2_press)
        self.bind('<ButtonRelease-2>', self._button2_release)
        self.bind('<B2-Motion>', self._button2_move)

        self.bind('<ButtonPress-3>', self._button3_press)
        self.bind('<ButtonRelease-3>', self._button3_release)

        self.bind('<Expose>', self._expose)

    def _button1_press(self, *args):
        pass

    def _button1_release(self, *args):
        pass

    def _button2(self, *args):
        pass

    def _button2_move(self, event):
        pass

    def _button2_press(self, *args):
        pass

    def _button2_release(self, *args):
        pass

    def _button3(self, *args):
        pass

    def _button3_press(self, *args):
        pass

    def _button3_release(self, *args):
        pass

    def _expose(self, *args):
        pass

    def _get_dpi(self, window):
        MM_TO_IN = 1/25.4
        pxw = window.winfo_screenwidth()
        inw = window.winfo_screenmmwidth() * MM_TO_IN
        return int(pxw/inw)

    def _mouse_move(self, *args):
        pass

    def _mouse_wheel(self, event):
        sf = 1.0
        if event.delta < 0:
            sf = 0.9
        else:
            sf = 1.3
        pass

    def on_exit(self):
        self.destroy()

    def _on_resize(self, event):
        """
        override from child.
        This event is hook <"Configure">
        If size is not changed, through the function.
        """
        #if (self.geometry != self.geometry()):
        #    self.geometry = self.geometry()
        self.on_resize(event)

    def on_resize(self, event):
        # override from child.
        pass

    def udp_receive_changed(self, *args):
        """
        When UDP use, then override this function .
        """
        pass

def main():
    root = Form()
    #root.wm_attributes("-transparentcolor", "white")
    root.mainloop()


if __name__ == '__main__':
    main()
